{
    "private": true,
    "type": "module",
    "scripts": {
        "dev": "vite",
        "watch": "vite build --watch",
        "build": "vite build{% if enable_ssr %} && vite build --ssr{% endif %}"
    },
    "dependencies": {{ dependencies }},
    "devDependencies": {{ dev_dependencies }}
}
