{
  "compilerOptions": {
    "target": "ES2020",
    "useDefineForClassFields": true,
    "module": "ES2020",
    "moduleResolution": "bundler",
    "strict": true,
    "jsx": "preserve",
    "resolveJsonModule": true,
    "isolatedModules": true,
    "esModuleInterop": true,
    "lib": ["ES2020", "DOM", "DOM.Iterable"],
    "skipLibCheck": true,
    "noEmit": true,
    "composite": true,
    "allowSyntheticDefaultImports": true,
    "baseUrl": "{{ resource_path }}",
    "paths": {
      "@/*": ["./*"]
    },
    "types": ["vite/client"]
  },
  "include": [
    {% if include_react %}"{{ resource_path }}/**/*.tsx","{{ resource_path }}/**/*.jsx",{% endif %}{% if include_vue %} "{{ resource_path }}/**/*.vue",{% endif %}
    "**/*.d.ts",
    "{{ resource_path }}/**/*.js" ,
    "{{ resource_path }}/**/*.ts" ,
    "vite.config.ts"
  ]
}
